package com.example.wluper_plugin_flutter

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import android.content.Context
import android.app.Activity
import androidx.annotation.NonNull
import com.wluper.sdk.WluperInterface
import com.wluper.sdk.WluperSDK
import com.wluper.wluper_sdk.models.WluperLog
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding


class WluperPluginFlutterPlugin: FlutterPlugin, MethodCallHandler, WluperInterface, ActivityAware {

  private lateinit var channel : MethodChannel
  private lateinit var wluper : WluperSDK
  private lateinit var context: Context
  private lateinit var activity: Activity
  private lateinit var x_api_key: String
  private lateinit var result_asr: Result
  private lateinit var result_tree: Result
  private var wluperLog : WluperLog = WluperLog()


  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "wluper_plugin_flutter")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {

    if (call.method == "WluperSDK") {
      x_api_key = (call.argument<String>("x_api_key"))!!
      wluper = WluperSDK(context, this as WluperInterface, x_api_key)
      result.success(0)
    }
    else if (call.method == "startListening") {
      result_asr = result
      wluper.startListening()
    }
    else if (call.method == "cancelListening") {
      wluper.cancelListening()
      result.success(0)
    }
    else if (call.method == "understand") {
      result_tree = result;
      var text = (call.argument<String>("plain_text"))!!
      wluper.wluperTU(text)
    }
    else if (call.method == "WluperDeviceInfo") {
      wluperLog.api_key = x_api_key
      wluperLog.device_id = (call.argument<String>("device_id"))!!
      wluperLog.model = (call.argument<String>("model"))!!
      wluperLog.os_name = (call.argument<String>("os_name"))!!
      wluperLog.os_version = (call.argument<String>("os_version"))!!
      wluperLog.sdk_version = (call.argument<String>("sdk_version"))!!
      wluper.setWluperLogBase(wluperLog)
      result.success(0)
    }
    else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun WluperLiveText(text: String) {
    channel.invokeMethod("WluperLiveText", text)
  }

  override fun WluperError(error: String) {
    channel.invokeMethod("WluperError", error)
  }

  override fun WluperTree(tree: Any) {
    result_tree.success(tree)
  }

  override fun WluperASR(plain_text: String) {
    result_asr.success(plain_text)
  }

  override fun WluperEndOfSpeech(){
    channel.invokeMethod("WluperEndOfSpeech", null)
  }
  

  override fun onDetachedFromActivity() {
    //TODO("Not yet implemented")
  }

  override fun onDetachedFromActivityForConfigChanges() {
    //TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

  }
  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }
}
